/*******************************************************************************
 *  Copyright 2017-2018 Centre for Development of Advanced Computing(C-DAC), Pune
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *   http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *******************************************************************************/
package cdac.medinfo.sdk.dcm30_04.commoninterface;

import java.io.IOException;

/**
 * This interface needs to be implemented by all classes which implements the necessary functionality for the Dicom File Writer
 */
public interface IDicomFileWriter
{
	/**
	 * Write the preamble value to file.
	 * @return int - shows status as success or failure.
	 * @throws IOException if Input/Output operation fails.
	 */
	int writePreamble() throws IOException;

	/**
	 * Write the preamble value to file. 
	 * @param preamble byte array containing preamble value.
	 * @return int - shows status as success or failure.
	 * @throws IOException if Input/Output operation fails.
	 */
	int writePreamble(byte[] preamble) throws IOException;

	/**
	 * Write the header prefix to the file.
	 * @return int - shows status as success or failure.
	 * @throws IOException if Input/Output operation fails.
	 */
	int writeHeaderPrefix() throws IOException;

	/**
	 * Write the header to the file.
	 * @param header byte array containing header value.
	 * @return int - shows status as success or failure.
	 * @throws IOException if Input/Output operation fails.
	 */
	int writeHeader(byte[] header) throws IOException;

	/**
	 * Write the header to the file.
	 * @param objDicomFileHeader instance of DicomFielHeader.
	 * @return int - shows status as success or failure.
	 * @throws IOException if Input/Output operation fails.
	 */
	int writeHeader(IDicomFileHeader objDicomFileHeader) throws IOException;

	/**
	 * Write the default header to the file.
	 * @return int - shows status as success or failure.
	 * @throws IOException if Input/Output operation fails.
	 */
	int writeDefaultHeader() throws IOException;

	/**
	 * Sets the header to the file.
	 * @param objDicomFileHeader_ instance of dicom file header.
	 * @return int - shows status as success or failure.
	 */
	int setHeader(IDicomFileHeader objDicomFileHeader_);

	/**
	 * Write the data contents to the file.
	 * @param dataContents Byte array containing DataContents value.
	 * @return int - shows status as success or failure.
	 * @throws IOException if Input/Output operation fails.
	 */
	int writeDataContents(byte[] dataContents) throws IOException;
}
